package de.moritzruth.lampenfieber

import de.moritzruth.lampenfieber.device.Tops
import de.moritzruth.lampenfieber.device.devices
import de.moritzruth.theaterdsl.dmx.EnttecOpenDmxUsb
import de.moritzruth.theaterdsl.dmx.PerDeviceDmxDataWriter
import de.moritzruth.theaterdsl.value.percent
import kotlinx.coroutines.coroutineScope
import kotlinx.coroutines.delay
import kotlinx.coroutines.isActive
import kotlinx.coroutines.launch
import kotlin.math.roundToLong
import kotlin.system.measureTimeMillis

suspend fun main() = coroutineScope {
    EnttecOpenDmxUsb.start()

    launch {
        test()
    }

    while (isActive) {
        val took = measureTimeMillis {
            val data = UByteArray(512)
            val writer = PerDeviceDmxDataWriter(data)

            for (device in devices) {
                writer.reset(0, device.firstChannel, device.numberOfChannels)
                device.writeDmxData(writer, false)
            }

            EnttecOpenDmxUsb.data = data
        }

        delay(((1000.0 / 30) - took).roundToLong())
    }
}

suspend fun test() = coroutineScope {
    Tops.both {
        it.brightness.static(100.percent)
        it.startRoomMovement(8.0)
    }
}