package de.moritzruth.lampenfieber.act

import de.moritzruth.lampenfieber.PropConfigurations
import de.moritzruth.lampenfieber.device.*
import de.moritzruth.theaterdsl.show.CurtainState
import de.moritzruth.theaterdsl.show.ShowBuilderContext
import de.moritzruth.theaterdsl.show.StepCue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.degrees
import de.moritzruth.theaterdsl.value.percent
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import kotlin.time.Duration.Companion.milliseconds
import kotlin.time.Duration.Companion.minutes
import kotlin.time.Duration.Companion.seconds

fun ShowBuilderContext.firstAct() = act("Erster Akt") {
    @Suppress("DuplicatedCode")
    scene("Intro") {
        step(StepCue.MusicStart("Lampenfieber", 5.minutes + 10.seconds)) {
            props(PropConfigurations::classroom)

            onRun {
                Washs.both.forEach {
                    it.pointAtCeiling()
                    it.colorWheelMode.static(CoemarProWash.ColorWheelMode.White)
                }

                Tops.both {
                    it.prismRotationSpeed.static(0.percent)
                }
            }
        }

        lightStep(StepCue.Custom("Gitarren-Einsatz (Takt 17)")) {
            Washs.both.forEach { it.brightness.pulseOnce(500.milliseconds, 5.seconds) }
            FrontLights.all.forEach { it.brightness.pulseOnce(500.milliseconds, 5.seconds, start = 20.percent) }
            Tops.both.forEach { it.startRoomMovement(4.5) }
        }

        lightStep(StepCue.Custom("Gitarren-Ton")) {
            launch {
                delay(500)
                Washs.both.forEach { it.brightness.pulseOnce(500.milliseconds, 5.seconds) }
            }

            Tops.both.forEach { it.brightness.pulseOnce(500.milliseconds, 5.seconds) }
        }

        lightStep(StepCue.Custom("Gitarren-Ton")) {
            Washs.both.forEach { it.brightness.pulseOnce(500.milliseconds, 5.seconds) }

            FrontLights.all.forEach { it.brightness.pulseOnce(500.milliseconds, 5.seconds, start = 20.percent) }
        }

        lightStep(StepCue.Custom("Gitarren-Ton")) {
            launch {
                delay(500)
                Washs.both.forEach { it.brightness.pulseOnce(500.milliseconds, 5.seconds) }
            }

            Tops.both.forEach { it.brightness.pulseOnce(500.milliseconds, 5.seconds) }
        }

        lightStep(StepCue.Custom("Gitarren-Ton")) {
            Washs.both.forEach { it.brightness.pulseOnce(500.milliseconds, 5.seconds) }

            FrontLights.all.forEach { it.brightness.pulseOnce(500.milliseconds, 5.seconds, start = 20.percent) }
        }

        lightStep(StepCue.Custom("Gitarren-Ton")) {
            launch {
                delay(500)
                Washs.both.forEach {
                    it.brightness.pulseOnce(500.milliseconds, 5.seconds)
                }
            }

            Tops.both.forEach { it.brightness.pulseOnce(500.milliseconds, 5.seconds) }
        }

        lightStep(StepCue.Custom("Gitarren-Ton")) {
            Washs.both.forEach { it.brightness.fade(100.percent, 500.milliseconds) }
        }

        lightStep(StepCue.Custom("Gitarren-Ton")) {
            Tops.both.forEach { it.brightness.fade(100.percent, 500.milliseconds) }
        }

        lightStep(StepCue.Custom("Gitarren-Ton")) {
            FrontLights.all.forEach { it.brightness.pulseOnce(500.milliseconds, 3.seconds, end = 50.percent) }
        }

        lightStep(StepCue.Custom("Gitarren-Ton")) {
            Washs.both.forEach { it.colorWheelMode.static(CoemarProWash.ColorWheelMode.Rotate(20.percent)) }
            Tops.both.forEach { it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.Rotate(10.percent)) }
        }

        step(StepCue.Text("Chor", "fangen wir an", "beim dritten Mal")) {
            onRun {
                FrontLights.center.forEach { it.brightness.fade(0.percent, 3.seconds) }
                Washs.both.forEach { it.brightness.fade(0.percent, 3.seconds) }
                Tops.both.forEach { it.brightness.fade(0.percent, 3.seconds) }
            }
        }
    }

    scene("Szene 1") {
        step(StepCue.MusicEnd) {
            actors {
                +"Richy / von rechts"
                +"Jakob / von rechts"
                +"Christine / steht auf der Vorbühne"
                +"Steffi / steht auf der Vorbühne"
                +"Andrea / steht auf der Vorbühne"
            }

            curtainState = CurtainState.OPEN

            onRun {
                FrontLights.center { it.brightness.fade(75.percent, 10.seconds) }

                Washs.both {
                    it.pointAtStageCenter()
                    it.colorWheelMode.static(CoemarProWash.ColorWheelMode.White)
                }
            }
        }

        step(StepCue.MusicStart("Rap", 2.minutes + 20.seconds)) {
            actors {
                +"Maria"
                +"Kathi"
                +"Paula"
                +"Sandra"
                +"Tina"
                +"David"
            }

            onRun {
                FrontLights.center.forEach { it.brightness.fade(20.percent, 10.seconds) }

                bar.brightness.fade(50.percent, 10.seconds)
                bar.color.fadeRandomAround(20.degrees, 10.degrees, 1500.milliseconds)

                sideLight.color3.fadeRandomAround(100.degrees, 10.degrees, 1500.milliseconds)
                sideLight.color4.fadeRandomAround(100.degrees, 10.degrees, 1500.milliseconds)
            }
        }

        lightStep(StepCue.Custom("Strophe")) {
            Washs.both { it.brightness.fade(100.percent, 200.milliseconds) }

            sideLight.brightness.fade(25.percent, 5.seconds)
        }

        lightStep(StepCue.Custom("Zwischenspiel (ohne Text)")) {
            Washs.both.forEach { it.brightness.off(200.milliseconds) }

            sideLight.brightness.off(1.seconds)
        }

        lightStep(StepCue.Custom("Strophe")) {
            Washs.both { it.brightness.fade(100.percent, 200.milliseconds) }

            sideLight.brightness.fade(25.percent, 1.seconds)
        }

        lightStep(StepCue.Custom("Schlussschlag")) {
            FrontLights.center.forEach { it.brightness.pulseOnce(200.milliseconds, 4.seconds, end = 50.percent) }

            Washs.both.forEach {
                it.beamAngle.fade(100.percent, 200.milliseconds)
                it.brightness.fade(0.percent, 4.seconds)
            }

            bar.brightness.off(1.seconds)
            sideLight.color3.fade(Color(100.degrees, brightness = 0.percent), 1.seconds)
            sideLight.color4.fade(Color(100.degrees, brightness = 0.percent), 1.seconds)
        }

        step(StepCue.MusicEnd) {
            actors {
                -"Christine"
                -"Andrea"
                -"Steffi"
                -"Maria"
                -"Kathi"
                -"Paula"
                -"Sandra"
                -"David"
                -"Jakob"
                -"Tina"
            }
        }

        step(StepCue.Text("Richy", "Tschau")) {
            actors {
                +"Tina / von rechts"
            }
        }

        step(StepCue.Text("Richy", "Tschüss")) {
            actors {
                -"Richy / nach links"
                -"Tina / nach links"
            }

            onRun {
                FrontLights.center { it.brightness.off(3.seconds) }
            }
        }
    }

    scene("Szene 2") {
        step(StepCue.Light(StepCue.Light.State.OFF, false)) {
            actors {
                +"David / sitzt am Klavier"
                +"Klavierlehrer / steht neben David"
            }
        }

        lightStep(StepCue.Custom("Umbau fertig")) {
            FrontLights.left.forEach { it.brightness.fade(75.percent, 10.seconds) }
            sideLight.brightness.static(100.percent)
            sideLight.fadeColorsToClassroomFromDarkness(10.seconds)
        }

        step(StepCue.Text("David", "Ende des Spielens")) {
            actors {
                -"Klavierlehrer / nach links"
            }

            onRun {
                FrontLights.left.forEach { it.brightness.off(5.seconds) }
                FrontLights.right.forEach { it.brightness.fade(50.percent, 5.seconds) }
            }
        }
    }

    scene("Szene 3") {
        step(StepCue.Custom("Auftritt im Mittelgang")) {
            actors {
                +"Tina / durch einen Gang"
                +"Maria / durch einen Gang"
                +"Heike / durch einen Gang"
                +"Sven / durch einen Gang"
                +"Sandra / durch einen Gang"
                +"Eva / durch einen Gang"
                +"Kathi / von der Bühne"
                +"Paula / steht rechts"
            }

            leftSpotTarget = "die Gruppe im linken Gang"
            rightSpotTarget = "die Gruppe im rechten Gang"

            onRun {
                spotLeft.brightness.fade(100.percent, 5.seconds)
                spotRight.brightness.fade(100.percent, 5.seconds)
            }
        }

        step(StepCue.Custom("Bühne erreicht")) {
            leftSpotTarget = null
            rightSpotTarget = null

            onRun {
                spotLeft.brightness.off(8.seconds)
                spotRight.brightness.off(8.seconds)
                FrontLights.all.forEach { it.brightness.fade(50.percent, 10.seconds) }
            }
        }
    }

    scene("Szene 5") {
        step(StepCue.Text("Maria", "Schreib mich auch dazu!")) {
            actors {
                -"Tina"
                -"Sven"
                -"Sandra"
                -"Heike"
                -"Maria"
                -"Eva"
                -"David"
                -"Kathi"
            }

            rightSpotTarget = "Paula"

            onRun {
                FrontLights.all.forEach { it.brightness.off(10.seconds) }
                spotRight.brightness.fade(100.percent, 5.seconds)
            }
        }

        step(StepCue.Text("Paula", "Die lachen doch alle über mich…")) {
            actors {
                +"Kai / von links"
                +"Adrian / von links"
            }

            onRun {
                FrontLights.center.forEach { it.brightness.fade(50.percent, 5.seconds) }

                Washs.both.forEach {
                    it.beamAngle.static(100.percent)
                    it.colorWheelMode.static(CoemarProWash.ColorWheelMode.Red)
                }
            }
        }

        step(StepCue.Custom("Kai und Adrian gehen ab")) {
            actors {
                -"Kai / nach links"
                -"Adrian / nach links"
            }

            onRun {
                FrontLights.center.forEach { it.brightness.off(10.seconds) }
            }
        }

        step(StepCue.MusicStart("Aus dem Schatten treten", 4.minutes)) {
            onRun {
                Washs.both.forEach { it.brightness.fade(100.percent, 10.seconds) }
                sideLight.colors.forEach { it.fade(Color(190.degrees, brightness = 50.percent), 10.seconds) }

                bar.color.static(Color(190.degrees))
                bar.brightness.fade(50.percent, 40.seconds)
            }
        }

        step(StepCue.MusicEnd) {
            actors {
                -"Paula / nach rechts"
            }

            rightSpotTarget = null

            onRun {
                FrontLights.right.forEach { it.brightness.off(5.seconds) }
                Washs.both.forEach { it.brightness.off(5.seconds) }
                spotRight.brightness.off(5.seconds)
                bar.brightness.off(5.seconds)
                sideLight.colors.forEach { it.fade(Color(190.degrees, brightness = 0.percent), 5.seconds) }
            }
        }
    }
}