package de.moritzruth.lampenfieber.act

import de.moritzruth.lampenfieber.PropConfigurations
import de.moritzruth.lampenfieber.device.*
import de.moritzruth.lampenfieber.song.songRichy
import de.moritzruth.lampenfieber.song.songSchauMichMit
import de.moritzruth.theaterdsl.show.CurtainState
import de.moritzruth.theaterdsl.show.ShowBuilderContext
import de.moritzruth.theaterdsl.show.StepCue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.degrees
import de.moritzruth.theaterdsl.value.percent
import kotlinx.coroutines.delay
import kotlinx.coroutines.isActive
import kotlinx.coroutines.launch
import kotlin.time.Duration
import kotlin.time.Duration.Companion.milliseconds
import kotlin.time.Duration.Companion.minutes
import kotlin.time.Duration.Companion.seconds

fun ShowBuilderContext.fourthAct() = act("Vierter Akt") {
    scene("Szene 1") {
        lightStep(StepCue.Custom("Pause")) {}

        lightStep(StepCue.MusicStart("Ende der Pause", 30.seconds)) {
            val sideColor = Color(60.degrees, saturation = 20.percent)
            sideLight.color1.static(sideColor)
            sideLight.color2.static(sideColor)
            sideLight.color3.static(Color.BLACK)
            sideLight.color4.static(Color.BLACK)
            sideLight.brightness.off()

            Washs.both {
                it.pointAtBathroomObjects()
                it.colorWheelMode.static(CoemarProWash.ColorWheelMode.White)
                it.beamAngle.static(100.percent)
            }

            Washs.left.magenta.static(100.percent)
            Washs.right.cyan.static(100.percent)
        }

        step(StepCue.MusicEnd) {
            actors {
                +"Sandra / sitzt links"
                +"Tina / sitzt rechts"
            }

            curtainState = CurtainState.OPEN

            onRun {
                val duration = 8.seconds
                sideLight.brightness.fade(100.percent, duration)
                Washs.both { it.brightness.fade(100.percent, duration) }
            }
        }

        lightStep(StepCue.Text("Sandra", "Das reicht nicht.")) {
            val duration = 2.seconds
            Washs.left.cyan.fade(100.percent, duration)
            Washs.left.magenta.fade(0.percent, duration)

            Washs.right.cyan.fade(0.percent, duration)
            Washs.right.magenta.fade(100.percent, duration)
        }

        step(StepCue.Text("Tina", "was ich anziehen soll.")) {
            actors {
                +"Manuel / von rechts"
            }
        }

        step(StepCue.Text("Tina", "Dann geh halt oben.")) {
            actors {
                -"Manuel / nach rechts"
                +"Bianca / von links"
            }

            onRun {
                val duration = 2.seconds
                Washs.left.cyan.fade(0.percent, duration)
                Washs.left.magenta.fade(100.percent, duration)

                Washs.right.cyan.fade(100.percent, duration)
                Washs.right.magenta.fade(0.percent, duration)
            }
        }

        step(StepCue.Text("Bianca", "Ich verzieh mich ja schon.")) {
            actors {
                -"Bianca / nach rechts"
            }

            onRun {
                // für den Song
                Tops.both {
                    it.startRoomMovement(10.0)
                    it.prismMode.static(FuturelightDmh160.PrismMode.FACETS_8)
                    it.prismRotationSpeed.static(20.percent)
                    it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.Yellow)
                }
            }
        }

        songSchauMichMit()

        step(StepCue.MusicStart("Umbaumusik", 2.minutes)) {
            props(PropConfigurations::richy)
        }
    }

    scene("Szene 2") {
        step(StepCue.MusicEnd) {
            actors {
                +"Tina / von der linken Treppe"
                +"Sandra / von der rechten Treppe"
            }

            onRun {
                // vor dem Vorhang
                FrontLights.all.forEach { it.brightness.fade(50.percent, 5.seconds) }
            }
        }

        step(StepCue.Text("Tina", "Macht auch Spaß. Echt!")) {
            actors {
                +"Kathi / von der linken Treppe"
                +"Maria / von der linken Treppe"
                +"Eva / von der linken Treppe"
            }
        }

        step(StepCue.Text("Tina", "Nö, ich bin ›Abseits‹.")) {
            actors {
                +"Richy / von der linken Treppe"
            }
        }

        step(StepCue.Text("Richy", "Hey, wartet doch!")) {
            actors {
                -"Kathi"
                -"Maria"
                -"Eva"
                -"Sandra"
                -"Richy"
            }
        }

        step(StepCue.MusicStart("Und jetzt", 5.minutes + 16.seconds)) {
            rightSpotTarget = "Tina"

            onRun {
                FrontLights.all.forEach { it.brightness.fade(25.percent, 12.seconds) }
                spotRight.brightness.fade(100.percent, 10.seconds)

                Washs.both {
                    it.pointAtGuests()
                    it.colorWheelMode.static(CoemarProWash.ColorWheelMode.Red)
                }
            }
        }

        lightStep(StepCue.Custom("1. Refrain")) {
            Washs.both {
                it.brightness.fade(100.percent, 3.seconds)
                it.pointAtCeiling(15.seconds)
            }
        }

        lightStep(StepCue.Custom("2. Strophe")) {
            val duration = 3.seconds
            Washs.both { it.brightness.off(duration) }

            launch {
                delay(duration)
                Washs.both { it.pointAtGuests() }
            }
        }

        lightStep(StepCue.Custom("2. Refrain")) {
            Washs.both {
                it.brightness.fade(100.percent, 3.seconds)
                it.pointAtCeiling(15.seconds)
            }
        }

        lightStep(StepCue.Custom("3. Strophe")) {
            val duration = 3.seconds
            Washs.both { it.brightness.off(duration) }

            launch {
                delay(duration)
                Washs.both { it.pointAtGuests() }
            }
        }

        lightStep(StepCue.Custom("3. Refrain")) {
            Washs.both {
                it.brightness.fade(100.percent, 3.seconds)
                it.pointAtCeiling(15.seconds)
            }
        }

        lightStep(StepCue.Custom("Ende des 3. Refrains")) {
            Washs.both { it.brightness.off(3.seconds) }
        }

        lightStep(StepCue.Custom("Letzte Töne")) {
            FrontLights.all.forEach { it.brightness.off(3.seconds) }
            spotRight.brightness.off(3.seconds)
        }

        step(StepCue.MusicEnd) {
            actors {
                -"Tina / durch die Mitte"
            }

            rightSpotTarget = null
        }
    }

    scene("Szene 3") {
        step(StepCue.MusicStart("Probenatmosphäre", 30.seconds)) {
            actors {
                +"David"
                // mehr
            }

            curtainState = CurtainState.OPEN

            onRun {
                FrontLights.all.forEach { it.brightness.fade(40.percent, 8.seconds) }

                bar.color.static(Color(50.degrees, saturation = 50.percent))
                bar.brightness.sine(6.seconds, minimum = 80.percent, maximum = 100.percent)

                sideLight.brightness.fade(100.percent, 8.seconds)
                sideLight.fadeColorsToClassroomFromDarkness(Duration.ZERO)
            }
        }
    }

    scene("Szene 4") {
        step(StepCue.MusicEnd) {
            actors {
                // alle ab außer David
                +"Paula / von links"
            }

            rightSpotTarget = "Paula und David (groß)"

            onRun {
                FrontLights.all.forEach { it.brightness.fade(20.percent, 6.seconds) }
                Washs.both.forEach {
                    it.pointAtCeiling()
                    it.colorWheelMode.static(CoemarProWash.ColorWheelMode.White)
                    it.magenta.static(100.percent)
                    it.beamAngle.sine(10.seconds, 20.percent, 80.percent)
                }

                bar.brightness.off(4.seconds)

                spotRight.brightness.fade(100.percent, 4.seconds)

                sideLight.color1.fade(Color(190.degrees, 75.percent), 45.seconds)
                sideLight.color3.fade(Color(320.degrees, 75.percent), 45.seconds)
                sideLight.brightness.fade(100.percent, 4.seconds)
            }
        }

        lightStep(StepCue.MusicStart("Wenn Du da bist (Duett)", 3.minutes + 5.seconds)) {
            Washs.both.forEach { it.brightness.fade(75.percent, 15.seconds) }
        }

        step(StepCue.MusicEnd) {
            actors {
                -"Paula"
                -"David"
            }

            rightSpotTarget = null

            onRun {
                spotRight.brightness.off(3.seconds)
                sideLight.brightness.off(8.seconds)

                FrontLights.all { it.brightness.off(3.seconds) }
                Washs.both { it.brightness.off(3.seconds) }
            }
        }
    }

    scene("Szene 5") {
        step(StepCue.Text("Richy", "Starke Frau!", "beim ersten Laut")) {
            actors {
                +"Richy / sitzt links"
                +"Eva / sitzt links"
                +"Maria / sitzt rechts"
                +"Sandra / sitzt rechts"
            }

            onRun {
                sideLight.fadeColorsToClassroomFromDarkness(5.seconds)
                sideLight.brightness.static(100.percent)

                FrontLights.all.forEach { it.brightness.fade(75.percent, 2.seconds) }
                Washs.both.forEach { it.brightness.off(5.seconds) }
            }
        }

        songRichy()
    }

    scene("Szene 6") {
        step(StepCue.MusicEnd) {
            props(PropConfigurations::classroom)

            actors {
                -"Richy"
                -"Sandra"
                -"Maria"
                -"Eva"
                +"Paula / von der linken Treppe"
                +"David / von der linken Treppe"
            }

            curtainState = CurtainState.CLOSED

            rightSpotTarget = "Paula & David"

            onRun {
                spotRight.brightness.fade(50.percent, 10.seconds)
                bar.color.static(Color(300.degrees, 70.percent))
                bar.brightness.fade(10.percent, 10.seconds)

                Washs.both.forEach {
                    it.beamAngle.static(75.percent)
                    it.brightness.fade(100.percent, 10.seconds)
                }
            }
        }

        step(StepCue.Custom("Ende der Hintergrundmusik")) {
            actors {
                +"Kai / von links"
                +"Adrian / von links"
            }

            leftSpotTarget = "Kai & Adrian"

            onRun {
                Washs.left.colorWheelMode.static(CoemarProWash.ColorWheelMode.Rotate(5.percent, true))
                spotLeft.brightness.static(100.percent)
            }
        }
    }

    scene("Szene 7") {
        step(StepCue.Text("David", "Aber auf jeden Fall nicht hier.")) {
            actors {
                +"Theaterlehrer"
                +"Sven"
                +"Heike"
                +"Andrea"
                +"Eva"
                +"Kathi"
                -"Kai"
                -"Adrian"
            }

            curtainState = CurtainState.OPEN

            leftSpotTarget = null
            rightSpotTarget = null

            onRun {
                // sideLights ist noch an
                FrontLights.all.forEach { it.brightness.fade(75.percent, 1.seconds) }

                Washs.both.forEach { it.brightness.off(1.seconds) }
                spotLeft.brightness.off(1.seconds)
                spotRight.brightness.off(1.seconds)
                bar.brightness.off(1.seconds)
            }
        }

        lightStep(StepCue.MusicStart("Paulas Monolog", 3.minutes + 8.seconds)) {
            Washs.both.forEach {
                it.beamAngle.static(100.percent)
                it.colorWheelMode.static(CoemarProWash.ColorWheelMode.DarkBlue)
            }

            Tops.both.forEach {
                it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.White)
                it.prismMode.static(FuturelightDmh160.PrismMode.FROST)
                it.prismRotationSpeed.static(0.percent)
                it.pointAtStageMiddleSingleCenterMarking()
            }

            FrontLights.all.forEach { it.brightness.fade(25.percent, 20.seconds) }
        }

        step(StepCue.Text("Paula", "Starrt mich nicht so an.", "auf das letzte Wort")) {
            rightSpotTarget = "Paula"

            onRun {
                Tops.both.forEach { it.brightness.pulseOnce(200.milliseconds, 10.seconds) }
                FrontLights.all.forEach { it.brightness.off() }

                spotRight.brightness.fade(50.percent, 6.seconds)

                bar.color.static(Color(190.degrees))
                bar.brightness.fade(start = 0.percent, end = 50.percent, duration = 3.seconds)

                sideLight.color1.static(Color(190.degrees, 40.percent, 40.percent))
                sideLight.color2.fadeRandomAround(190.degrees, 10.degrees, 5.seconds)
                sideLight.color3.static(Color(190.degrees, 40.percent, 70.percent))
                sideLight.color4.fadeRandomAround(190.degrees, 20.degrees, 6.seconds)

                Washs.both.forEach {
                    it.brightness.fade(100.percent, 3.seconds)
                    it.beamAngle.fade(0.percent, 2.seconds)
                }
            }
        }

        lightStep(StepCue.Custom("Keyboard-Einsatz")) {
            Tops.both.forEach {
                it.startRoomMovement(3.0)
                it.brightness.fade(100.percent, 10.seconds)
                it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.DarkBlue)
                it.prismRotationSpeed.static(20.percent)
            }
        }

        lightStep(StepCue.Text("Paula", "Ihr könnt euch nicht entscheiden")) {
            Tops.both.forEach { it.brightness.off(10.seconds) }
            bar.brightness.off(10.seconds)
            sideLight.brightness.off(10.seconds)
            spotRight.brightness.fade(100.percent, 10.seconds)
        }

        step(StepCue.MusicEnd) {
            props(PropConfigurations::backstage)

            actors {
                -"Paula"
                -"David"
                -"Sven"
                -"Heike"
                -"Andrea"
                -"Eva"
                -"Kathi"
                -"Theaterlehrer"
            }

            curtainState = CurtainState.CLOSED

            rightSpotTarget = null

            onRun {
                spotRight.brightness.off(5.seconds)
                Washs.both {
                    it.brightness.off(5.seconds)
                }
            }
        }
    }

    scene("Szene 8") {
        step(StepCue.MusicStart("Angstballett", 2.minutes + 20.seconds)) {
            onRun {
                // instrumental
            }
        }
    }

    scene("Szene 9") {
        step(StepCue.MusicEnd) {
            actors {
                +"Kai / durchs Publikum"
                +"Adrian / durchs Publikum"
            }

            rightSpotTarget = "Kai, Adrian"

            onRun {
                spotRight.brightness.static(100.percent)

                // für später
                Tops.both.forEach {
                    it.pointAtStageMiddleSingleCenterMarking()
                    it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.LightBlue)
                    it.prismRotationSpeed.static(0.percent)
                }
            }
        }
    }

    scene("Szene 10") {
        step(StepCue.Text("Kai", "Hilfe, los, runter!")) {
            curtainState = CurtainState.OPEN

            rightSpotTarget = null

            onRun {
                spotRight.brightness.off(5.seconds)
            }
        }

        step(StepCue.Curtain(CurtainState.OPEN, false)) {
            actors {
                +"Paula / steht bei Markierung"
            }

            onRun {
                Tops.both.forEach { it.brightness.fade(10.percent, 1.5.seconds) }

                bar.color.static(Color(245.degrees, 30.percent))
                sideLight.colors.forEach { it.static(Color.WHITE) }

//                val steps = MutableList(10) { 0.percent }
//                steps[0] = 100.percent
//                bar.brightness.steps(steps, 1.8.seconds / steps.size)

                launch {
                    while (isActive) {
                        val upDuration = 50.milliseconds
                        val downDuration = 200.milliseconds

                        bar.brightness.pulseOnce(upDuration, downDuration, peakValue = 30.percent)
                        delay(upDuration + downDuration)

                        sideLight.brightness.pulseOnce(upDuration, downDuration, peakValue = 10.percent)
                        delay(1.5.seconds - (upDuration + downDuration))
                    }
                }
            }
        }

        step(StepCue.MusicStart("Gemeinsam packen wir es an", 5.minutes + 25.seconds)) {
            onRun {
                Washs.both {
                    it.pointAtGuests()
                    it.colorWheelMode.static(CoemarProWash.ColorWheelMode.WarmWhite)
                }
            }
        }

        // 0:13
        lightStep(StepCue.Custom("Kick-Drum")) {
            sideLight.brightness.pulseOnce(50.milliseconds, 200.milliseconds, peakValue = 50.percent)
        }

        lightStep(StepCue.Custom("Kick-Drum")) {
            sideLight.brightness.pulseOnce(50.milliseconds, 200.milliseconds, peakValue = 10.percent)
        }

        lightStep(StepCue.Custom("Kick-Drum")) {
            sideLight.brightness.pulseOnce(50.milliseconds, 200.milliseconds, peakValue = 30.percent)
        }

        lightStep(StepCue.Custom("Kick-Drum")) {
            sideLight.brightness.pulseOnce(50.milliseconds, 200.milliseconds, peakValue = 50.percent)
        }

        lightStep(StepCue.Custom("Kick-Drum")) {
            sideLight.brightness.pulseOnce(50.milliseconds, 200.milliseconds, peakValue = 70.percent)
        }

        // 0:19
        lightStep(StepCue.Custom("Tusch")) {
            sideLight.brightness.pulseOnce(50.milliseconds, 200.milliseconds, peakValue = 90.percent)

            Washs.both { it.brightness.pulseOnce(50.milliseconds, 2.seconds) }

            Tops.both { it.brightness.off(1.seconds) }
        }

        // 0:29
        lightStep(StepCue.Text("Paula", "Da ist eine Tür", "auf das letzte Wort")) {
            sideLight.colors.forEach { it.static(Color.BLACK) }
            sideLight.color3.static(Color(245.degrees))
            sideLight.brightness.fade(100.percent, 2.seconds)
        }

        // 1:05
        lightStep(StepCue.Text("Paula", "Geh über die Schwelle", "auf die letzte Silbe")) {
            sideLight.brightness.off(2.seconds)
        }

        // 1:07
        lightStep(StepCue.Custom("Becken")) {
            Washs.both { it.brightness.pulseOnce(50.milliseconds, 2.seconds) }
        }

        lightStep(StepCue.Custom("2 Becken")) {
            bar.brightness.pulseOnce(50.milliseconds, 500.milliseconds)

            sideLight.colors.forEach { it.static(Color(245.degrees)) }

            launch {
                delay(200.milliseconds)
                sideLight.brightness.pulseOnce(50.milliseconds, 300.milliseconds)
            }
        }

        repeat(2) {
            lightStep(StepCue.Custom("Becken")) {
                Washs.both { it.brightness.pulseOnce(50.milliseconds, 2.seconds) }
            }

            lightStep(StepCue.Custom("2 Becken")) {
                bar.brightness.pulseOnce(50.milliseconds, 500.milliseconds)

                launch {
                    delay(200.milliseconds)
                    sideLight.brightness.pulseOnce(50.milliseconds, 300.milliseconds)
                }
            }
        }

        // 1:19
        lightStep(StepCue.Custom("Beginn der Steigerung")) {
            Washs.both {
                it.brightness.fade(100.percent, 50.milliseconds)
                it.pointAtCeiling(6.seconds)
            }
        }

        // 1:25
        lightStep(StepCue.Custom("Becken")) {
            Washs.both { it.brightness.off(3.seconds) }
        }

        // 1:35
        step(StepCue.Text("Paula", "Aus dem Schatten treten")) {
            rightSpotTarget = "Paula"

            onRun {
                spotRight.brightness.fade(50.percent, 5.seconds)
            }
        }

        // 1:51
        lightStep(StepCue.Custom("Einsatz der anderen")) {
            FrontLights.all { it.brightness.fade(100.percent, 30.seconds) }

            Tops.both {
                it.startRoomMovement(5.0)
                it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.Rotate(10.percent))
                it.prismMode.static(FuturelightDmh160.PrismMode.FROST)
            }

            Washs.both { it.colorWheelMode.static(CoemarProWash.ColorWheelMode.Rotate(50.percent)) }
        }

        lightStep(StepCue.Custom("Verbeugung")) {
            Tops.both { it.brightness.fade(100.percent, 2.seconds) }
            Washs.both { it.brightness.fade(100.percent, 2.seconds) }
        }

        lightStep(StepCue.Custom("Ende")) {
            spotRight.brightness.off(1.seconds)
            Tops.both { it.brightness.off(1.seconds) }
            Washs.both { it.brightness.off(1.seconds) }
            FrontLights.all { it.brightness.off(1.seconds) }
        }
    }
}