package de.moritzruth.lampenfieber.act

import de.moritzruth.lampenfieber.PropConfigurations
import de.moritzruth.lampenfieber.device.*
import de.moritzruth.theaterdsl.show.CurtainState
import de.moritzruth.theaterdsl.show.ShowBuilderContext
import de.moritzruth.theaterdsl.show.StepCue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.degrees
import de.moritzruth.theaterdsl.value.percent
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import kotlin.time.Duration.Companion.milliseconds
import kotlin.time.Duration.Companion.minutes
import kotlin.time.Duration.Companion.seconds

fun ShowBuilderContext.secondAct() = act("Zweiter Akt") {
    scene("Szene 1") {
        step(StepCue.MusicStart("Umbaumusik", 1.minutes + 30.seconds)) {
            props(PropConfigurations::tvRoom)
        }
    }

    scene("Szene 2") {
        step(StepCue.MusicEnd) {
            actors {
                +"Heike / von links"
                +"Sven / steht rechts"
            }

            onRun {
                FrontLights.all.forEach { it.brightness.fade(50.percent, 8.seconds) }
            }
        }
    }

    scene("Szene 3") {
        step(StepCue.Text("Sven", "Okay!")) {
            actors {
                -"Heike"
                -"Sven"
                +"Paula / sitzt auf Sofa"
            }

            curtainState = CurtainState.OPEN

            onRun {
                (FrontLights.left + FrontLights.right).forEach { it.brightness.off(5.seconds) }

                // Fernsehsimulation
                bar.color.fadeRandomAround(180.degrees, 90.degrees, 2000.milliseconds)
                bar.brightness.fade(20.percent, 5.seconds)
            }
        }

        step(StepCue.Text("Paula", "Sterilux, was ist denn das?", "1s nach Ende")) {
            actors {
                +"Ajax / von links"
                +"Lenora / von links"
                +"Leila / von links"
                +"Proper / von links"
                +"Astronaut / liegt in der Mitte der Bühne"
            }

            rightSpotTarget = "Paula"

            onRun {
                FrontLights.center.forEach { it.brightness.off() }
                spotRight.brightness.static(20.percent)
                bar.color.static(Color(140.degrees))

                sideLight.brightness.static(100.percent)
                sideLight.color1.fadeRandomAround(300.degrees, 10.degrees, 5.seconds)
                sideLight.color2.fadeRandomAround(140.degrees, 10.degrees, 6.seconds)
                sideLight.color3.fadeRandomAround(235.degrees, 10.degrees, 7.seconds)
                sideLight.color4.fadeRandomAround(235.degrees, 10.degrees, 8.seconds)
            }
        }

        step(StepCue.Text("Lenora", "da kommt der General")) {
            actors {
                +"General / von links"
            }
        }

        step(StepCue.Text("Astronaut", "Ich hau ab!")) {
            actors {
                -"Ajax / nach links"
                -"Lenora / nach links"
                -"Leila / nach links"
                -"Proper / nach links"
                -"General / nach links"
                -"Astronaut / nach links"
            }

            onRun {
                spotRight.brightness.fade(100.percent, 5.seconds)
                bar.brightness.off(10.seconds)
                // sideLight bleibt an
            }
        }

        step(StepCue.MusicStart("Eine Rolle spielen I", 2.minutes + 45.seconds)) {
            Washs.both.forEach {
                it.colorWheelMode.static(CoemarProWash.ColorWheelMode.WarmWhite)
                it.pointAtGuests()
            }
        }

        lightStep(StepCue.Custom("1. Refrain")) {
            Tops.both.forEach {
                it.brightness.fade(20.percent, 5.seconds)
                it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.Orange)
                it.prismMode.static(FuturelightDmh160.PrismMode.FROST)
                it.startRoomMovement(3.0)
            }
        }

        lightStep(StepCue.Text("Paula", "Hier steh nur ich.", "auf das letzte Wort")) {
            Washs.both.forEach {
                it.pointAtCeiling(8.seconds)
                it.brightness.fade(100.percent, 2.seconds)
            }
        }

        lightStep(StepCue.Text("Paula", "wer oben schwimmt.", "auf das letzte Wort")) {
            val duration = 1.5.seconds
            Washs.both.forEach { it.brightness.off(1.5.seconds) }

            launch {
                delay(duration)
                Washs.both.forEach { it.pointAtGuests() }
            }
        }

        lightStep(StepCue.Custom("Refrain")) {
            Washs.both.forEach {
                it.pointAtCeiling(8.seconds)
                it.brightness.fade(100.percent, 2.seconds)
            }
        }

        lightStep(StepCue.Text("Paula", "mal grässlich.", "im letzten Refrain")) {
            Washs.both.forEach { it.brightness.off(3.seconds) }
            Tops.both.forEach {it.brightness.off(3.seconds) }
        }
    }

    scene("Szene 4") {
        step(StepCue.MusicEnd) {
            onRun {
                Tops.both.reset()
            }
        }

        step(StepCue.Text("Paula", "spielt keine Rolle.")) {
            actors {
                -"Paula"
            }

            rightSpotTarget = null

            curtainState = CurtainState.CLOSED

            onRun {
                spotRight.brightness.off(3.seconds)
                sideLight.brightness.off(3.seconds)
            }
        }

        step(StepCue.MusicStart("Eine Rolle spielen II", 2.minutes)) {
            props(PropConfigurations::casting)
            // instrumental
        }
    }
}