package de.moritzruth.lampenfieber.act

import de.moritzruth.lampenfieber.PropConfigurations
import de.moritzruth.lampenfieber.device.*
import de.moritzruth.theaterdsl.device.reset
import de.moritzruth.theaterdsl.show.CurtainState
import de.moritzruth.theaterdsl.show.ShowBuilderContext
import de.moritzruth.theaterdsl.show.StepCue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.degrees
import de.moritzruth.theaterdsl.value.percent
import kotlin.time.Duration
import kotlin.time.Duration.Companion.milliseconds
import kotlin.time.Duration.Companion.minutes
import kotlin.time.Duration.Companion.seconds

@Suppress("DuplicatedCode")
fun ShowBuilderContext.thirdAct() = act("Dritter Akt") {
    scene("Szene 1") {
        step(StepCue.MusicEnd) {
            actors {
                +"Jakob / durch den linken Gang"
                +"Andrea / durch den linken Gang"
                +"Domina / durch den rechten Gang"
                +"Valentina / durch den rechten Gang"
            }

            leftSpotTarget = "Jakob und Andrea / im linken Gang"
            rightSpotTarget = "Domina und Valentina / im rechten Gang"

            onRun {
                spotLeft.brightness.fade(100.percent, 4.seconds)
                spotRight.brightness.fade(100.percent, 4.seconds)

                sideLight.colors.forEach { it.static(Color.BLACK) }
                sideLight.color1.static(Color(54.degrees))
                sideLight.brightness.static(100.percent)
            }
        }

        step(StepCue.Custom("Erreichen der Bühne")) {
            actors {
                +"Sandra / steht links"
                +"Richy / steht links"
                +"Maria / hinter der rechten Wand"
            }

            leftSpotTarget = null
            rightSpotTarget = null

            curtainState = CurtainState.OPEN

            onRun {
                spotLeft.brightness.off(5.seconds)
                spotRight.brightness.off(5.seconds)

                FrontLights.all { it.brightness.fade(75.percent, 6.seconds) }
            }
        }
    }

    scene("Szene 2") {
        step(StepCue.Text("Richy", "Aber ich kann ja nicht überall sein.")) {
            actors {
                -"Richy / nach rechts"
                -"Maria / nach rechts"
                -"Sandra / nach rechts"
                -"Andrea"
                -"Domina"
                -"Valentina"
                -"Jakob" // TODO: Die vier wohin?

                +"David / am Klavier"
            }

            leftSpotTarget = "David"

            onRun {
                sideLight.brightness.off(10.seconds)
                spotLeft.brightness.fade(100.percent, 5.seconds)
                FrontLights.all.forEach { it.brightness.fade(25.percent, 6.seconds) }
            }
        }

        step(StepCue.Custom("David spielt zum dritten Mal")) {
            actors {
                +"Paula / von rechts"
            }

            rightSpotTarget = "Paula"

            onRun {
                spotRight.brightness.fade(100.percent, 10.seconds)

                Washs.right.pointAtStageCenter()
                Washs.right.colorWheelMode.static(CoemarProWash.ColorWheelMode.White)
                Washs.right.magenta.static(100.percent)
            }
        }

        lightStep(StepCue.MusicStart("Wenn du da bist (Klavierversion)", 1.minutes + 5.seconds)) {
            Washs.right.brightness.fade(100.percent, 15.seconds)
        }

        lightStep(StepCue.MusicEnd) {
            Washs.right.brightness.off(5.seconds)
        }

        step(StepCue.Text("David", "traust du dir das auch zu?")) {
            curtainState = CurtainState.CLOSED

            onRun {
                FrontLights.all.forEach { it.brightness.off() }
                Washs.right.magenta.reset()

                spotLeft.brightness.off(5.seconds)
                spotRight.brightness.off(5.seconds)
            }
        }

        step(StepCue.Curtain(CurtainState.CLOSED, false)) {
            props(PropConfigurations::classroom)

            actors {
                -"David"
                -"Paula"
            }

            leftSpotTarget = null
            rightSpotTarget = null
        }
    }

    scene("Szene 3") {
        lightStep(StepCue.MusicStart("Tischballett (Umbau)", 50.seconds)) {
            sideLight.fadeColorsToClassroomFromDarkness(Duration.ZERO)
        }
    }

    scene("Szene 4") {
        step(StepCue.MusicEnd) {
            actors {
                +"David"
                +"Paula"
                +"Kathi"
                +"Maria"
                +"Sandra"
                // mehr
            }

            curtainState = CurtainState.OPEN

            onRun {
                FrontLights.all.forEach { it.brightness.fade(75.percent, 8.seconds) }
                sideLight.brightness.fade(75.percent, 8.seconds)
            }
        }

        step(StepCue.Text("David", "Das baut einen so richtig auf.")) {
            actors {
                -"Paula / rennt"
            }
        }

        step(StepCue.Text("Gerda", "Ich glaub, wir ziehen ab.")) {
            actors {
                -"Maria"
                -"Sandra"
                // mehr
            }
        }
    }

    scene("Szene 5") {
        step(StepCue.Custom("Kathi ruft Paula")) {
            actors {
                +"Paula"
            }
        }

        step(StepCue.Text("Kathi", "Ciao.")) {
            actors {
                -"Kathi / nach links"
            }
        }

        step(StepCue.Text("David", "Echt nicht.")) {
            actors {
                +"Kai"
                +"Adrian"
            }
        }

        step(StepCue.Text("Kai", "ich bin auf allen Kanälen.")) {
            actors {
                -"Kai"
                -"Adrian"
                -"Paula"
                -"David"
            }

            curtainState = CurtainState.CLOSED

            onRun {
                FrontLights.all.forEach { it.brightness.fade(0.percent, 5.seconds) }
                sideLight.brightness.off(8.seconds)
            }
        }
    }

    scene("Szene 6") {
        step(StepCue.MusicStart("Umbaumusik", 2.minutes + 20.seconds)) {
            props(PropConfigurations::calling)

            onRun {
                Tops.both.forEach {
                    it.prismMode.static(FuturelightDmh160.PrismMode.OPEN)
                    it.prismRotationSpeed.off()
                    it.pointAtStageMiddleOffCenterMarking()
                    it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.White)
                }
            }
        }

        step(StepCue.MusicEnd) {
            actors {
                +"Kathi / sitzt links"
                +"Maria / sitzt rechts"
            }

            curtainState = CurtainState.OPEN
        }

        lightStep(StepCue.Text("Kathi", "Hallo, Markus", "beim ersten Laut")) {
            Tops.left.brightness.fade(100.percent, 3.seconds)
        }

        lightStep(StepCue.Text("Maria", "Hallo, Ingo", "beim ersten Laut")) {
            Tops.right.brightness.fade(100.percent, 3.seconds)
        }

        // songRopeSkippingTanz()

        step(StepCue.MusicStart("Rope-Skipping-Tanz", 3.minutes)) {
            onRun {
                FrontLights.all.forEach { it.brightness.fade(50.percent, 30.seconds) }
            }
        }

        lightStep(StepCue.Custom("Schlussschlag")) {
            FrontLights.all { it.brightness.pulseOnce(0.milliseconds, 2.seconds) }
        }
    }

    scene("Szene 7") {
        step(StepCue.MusicEnd) {
            actors {
                +"Sven"
            }

            rightSpotTarget = "die Mitte der Bühne"

            onRun {
                spotRight.brightness.static(100.percent)
            }
        }

        step(StepCue.Text("Sven", "den nicht zusammen machen?")) {
            actors {
                -"Sven"
                +"Tina"
            }
        }


        step(StepCue.Text("Tina", "Hör schon auf…")) {
            actors {
                -"Tina"
                +"Eva"
            }
        }

        step(StepCue.MusicStart("Terminhektik", 1.minutes + 10.seconds)) {
            actors {
                -"Eva"
            }

            rightSpotTarget = null

            onRun {
                FrontLights.all.forEach { it.brightness.fade(75.percent, 1.minutes, start = 20.percent) }
                spotRight.brightness.off()
            }
        }
    }

    scene("Szene 8") {
        step(StepCue.MusicEnd) {
        }

        step(StepCue.Text("Kathi", "nie was draus werden.")) {
            actors {
                +"David / von links"
            }
        }

        step(StepCue.MusicStart("Aus dem Schatten treten II", 3.minutes + 20.seconds)) {
            actors {
                -"Kathi"
                -"David"
            }

            rightSpotTarget = "Maria"

            onRun {
                FrontLights.all.forEach { it.brightness.fade(0.percent, 20.seconds) }

                spotRight.brightness.fade(50.percent, 10.seconds)

                sideLight.colors.forEach { it.static(Color(240.degrees)) }
                sideLight.brightness.fade(start = 0.percent, end = 100.percent, duration = 30.seconds)

                Tops.both.forEach {
                    it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.DarkBlue)
                    it.prismMode.static(FuturelightDmh160.PrismMode.FROST)
                    it.startRoomMovement(8.0)
                }
            }
        }

        step(StepCue.Custom("Paula tritt auf")) {
            actors {
                +"Paula / von links"
            }

            leftSpotTarget = "Paula"

            onRun {
                spotLeft.brightness.fade(50.percent, 2.seconds)
                sideLight.colors.forEach { it.fade(Color(185.degrees), 30.seconds) }

                Tops.both.forEach { it.brightness.fade(100.percent, 30.seconds) }
            }
        }

        lightStep(StepCue.Custom("Schlussschlag")) {
            Tops.both.forEach { it.brightness.off(1.seconds) }
            sideLight.colors.forEach { it.fade(Color(185.degrees, brightness = 0.percent), 1.seconds) }
            spotLeft.brightness.off()
            spotRight.brightness.off()
        }

        step(StepCue.MusicEnd) {
            props(PropConfigurations::bathroom)

            actors {
                -"Paula"
                -"Maria"
                +"Heike / aus dem Off"
                +"Sven / aus dem Off"
                +"Eva / aus dem Off"
                +"Sandra / aus dem Off"
                +"Paula (Off) / aus dem Off"
            }

            rightSpotTarget = null
            leftSpotTarget = null

            curtainState = CurtainState.CLOSED

            onRun {
                Tops.both { it.reset() }
            }
        }

        step(StepCue.MusicStart("Pause", 40.seconds)) {
            actors {
                -"Heike"
                -"Sven"
                -"Eva"
                -"Sandra"
                -"Paula"
            }
        }
        step(StepCue.MusicEnd) {}

        step(StepCue.Custom("Pause-Schild tritt auf")) {
            rightSpotTarget = "Pause-Schild"

            onRun {
                spotRight.brightness.fade(100.percent, 2.seconds)
            }
        }

        step(StepCue.Custom("Pause-Schild hat die Bühne überquert")) {
            rightSpotTarget = null

            onRun {
                spotRight.brightness.off(2.seconds)
            }
        }
    }
}