package de.moritzruth.lampenfieber.device

import de.moritzruth.theaterdsl.device.ColorDV
import de.moritzruth.theaterdsl.device.Device
import de.moritzruth.theaterdsl.device.PercentageDV
import de.moritzruth.theaterdsl.dmx.DmxAddress
import de.moritzruth.theaterdsl.dmx.DmxDataWriter
import de.moritzruth.theaterdsl.dmx.DmxValue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.degrees
import de.moritzruth.theaterdsl.value.percent
import kotlinx.collections.immutable.persistentListOf
import kotlinx.collections.immutable.persistentSetOf
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.delay
import kotlinx.coroutines.isActive
import kotlinx.coroutines.launch
import kotlin.random.Random
import kotlin.time.Duration
import kotlin.time.Duration.Companion.milliseconds

class StairvilleClb4(override val firstChannel: DmxAddress) : Device {
    override val numberOfChannels: UInt = 14u

    override fun writeDmxData(writer: DmxDataWriter, isLightBehindCurtainOn: Boolean) {
        if (isLightBehindCurtainOn) {
            repeat(3) {
                writer.writeRaw(DmxValue(255u))
                writer.writeRaw(DmxValue(255u))
                writer.writeRaw(DmxValue(255u))
            }

            writer.writeRaw(DmxValue(0u))
            writer.writeRaw(DmxValue(0u))
            writer.writeRaw(DmxValue(0u))

            writer.writeRaw(DmxValue(0u))
            writer.writeRaw(DmxValue(40u))

            return
        }

        for (color in colors) {
            val (red, green, blue) = color.getCurrentValue().getRGB()
            writer.writePercentage(red)
            writer.writePercentage(green)
            writer.writePercentage(blue)
        }

        writer.writePercentage(strobeSpeed.getCurrentValue())
        writer.writePercentage(brightness.getCurrentValue())
    }

    val brightness = PercentageDV(0.percent)
    val strobeSpeed = PercentageDV()

    val color1 = ColorDV(Color(0.degrees, brightness = 0.percent))
    val color2 = ColorDV(Color(0.degrees, brightness = 0.percent))
    val color3 = ColorDV(Color(0.degrees, brightness = 0.percent))
    val color4 = ColorDV(Color(0.degrees, brightness = 0.percent))
    val colors = persistentListOf(color1, color2, color3, color4)

    override val dvs = persistentSetOf(
        brightness,
        strobeSpeed,
        *colors.toTypedArray()
    )

    private val classroomColor = Color(40.degrees, saturation = 90.percent, brightness = 100.percent)

    fun fadeColorsToClassroomFromDarkness(duration: Duration) {
        sideLight.colors.forEach { it.static(classroomColor.copy(brightness = 0.percent)) }
        fadeColorsToClassroom(duration)
    }

    fun fadeColorsToClassroom(duration: Duration) {
        sideLight.color1.fade(color1.getCurrentValue().copy(brightness = 0.percent), duration)

        sideLight.color2.fade(classroomColor, duration = duration)
        sideLight.color3.fade(classroomColor, duration = duration)
        sideLight.color4.fade(classroomColor, duration = duration)
    }

    context(CoroutineScope)
    fun moveColorsPinkAndBlue() {
        val pink = Color(320.degrees)
        val blue = Color(220.degrees)

        colors.forEachIndexed { index, color ->
            launch {
                if (index % 2 == 0) {
                    val duration = Random.nextInt(500, 3000).milliseconds
                    color.fade(blue, duration)
                    delay(duration)
                }

                while (isActive) {
                    val firstDuration = Random.nextInt(500, 3000).milliseconds
                    color.fade(pink, firstDuration)
                    delay(firstDuration)

                    val secondDuration = Random.nextInt(500, 3000).milliseconds
                    color.fade(blue, secondDuration)
                    delay(secondDuration)
                }
            }
        }
    }
}