package de.moritzruth.lampenfieber.song

import de.moritzruth.lampenfieber.device.CoemarProWash
import de.moritzruth.lampenfieber.device.FrontLights
import de.moritzruth.lampenfieber.device.Washs
import de.moritzruth.lampenfieber.device.bar
import de.moritzruth.theaterdsl.show.CurtainState
import de.moritzruth.theaterdsl.show.SceneBuilderContext
import de.moritzruth.theaterdsl.show.StepCue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.degrees
import de.moritzruth.theaterdsl.value.percent
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import kotlin.time.Duration.Companion.milliseconds
import kotlin.time.Duration.Companion.minutes
import kotlin.time.Duration.Companion.seconds

fun SceneBuilderContext.songRichy() {
    step(StepCue.MusicStart("Richy", 3.minutes)) {
        onRun {
            Washs.both {
                it.magenta.static(0.percent)
                it.pointAtCeiling()
                it.beamAngle.fade(100.percent, 20.seconds, start = 0.percent)
                it.brightness.fade(100.percent, 10.seconds)
            }

            FrontLights.all { it.brightness.fade(40.percent, 15.seconds) }
        }
    }

    lightStep(StepCue.Text("Mädchen", "Ach, ist es nicht fürchterlich")) {
        Washs.both.forEach {
            it.pointAtGuests()
            it.brightness.off(1.seconds)
        }
    }

    // 0:40
    lightStep(StepCue.Custom("Schlagzeug-Einsatz")) {
        Washs.both.forEach {
            it.brightness.static(100.percent)
            it.strobeSpeed.static(50.percent)
        }
    }

    lightStep(StepCue.Custom("Nächster Takt-Anfang")) {
        Washs.both.forEach {
            it.strobeSpeed.static(0.percent)
            it.pointAtCeiling()
        }

        Washs.left.colorWheelMode.static(CoemarProWash.ColorWheelMode.Rotate(30.percent, false))
        Washs.right.colorWheelMode.static(CoemarProWash.ColorWheelMode.Rotate(30.percent, true))
    }

    // 1:00
    lightStep(StepCue.Custom("Crash-Becken")) {
        Washs.both.forEach { it.brightness.off(200.milliseconds) }
    }

    lightStep(StepCue.Custom("+2 Takte")) {
        bar.brightness.sine(4.seconds, 20.percent, 50.percent)
        bar.color.static(Color(40.degrees))

        Washs.both {
            it.brightness.fade(100.percent, 200.milliseconds)
            it.beamAngle.sine(4.seconds, start = 100.percent)
        }
    }

    // 1:24
    lightStep(StepCue.Custom("Crash-Becken")) {
        Washs.both { it.brightness.off(200.seconds) }
    }

    lightStep(StepCue.Custom("+1 Takt")) {
        Washs.both {
            it.pointAtGuests()
            it.beamAngle.fade(100.percent, 500.milliseconds)
            it.colorWheelMode.static(CoemarProWash.ColorWheelMode.WarmWhite)
        }
    }

    lightStep(StepCue.Custom("Schlagzeug-Einsatz")) {
        Washs.both {
            it.brightness.static(100.percent)
            it.strobeSpeed.static(50.percent)
        }
    }

    // 1:28
    lightStep(StepCue.Custom("Nächster Takt-Anfang")) {
        Washs.both {
            it.brightness.pulseOnce(200.milliseconds, 1.seconds)
            it.strobeSpeed.static(0.percent)
        }

        bar.brightness.pulseOnce(200.milliseconds, 1.seconds)
    }

    // 1:51
    lightStep(StepCue.Custom("Schlagzeug-Einsatz")) {
        Washs.both {
            it.brightness.static(100.percent)
            it.strobeSpeed.static(50.percent)
        }
    }

    lightStep(StepCue.Custom("Nächster Takt-Anfang")) {
        Washs.both {
            it.strobeSpeed.static(0.percent)
            it.pointAtCeiling()
        }

        Washs.left.colorWheelMode.static(CoemarProWash.ColorWheelMode.Rotate(30.percent, false))
        Washs.right.colorWheelMode.static(CoemarProWash.ColorWheelMode.Rotate(30.percent, true))
    }

    lightStep(StepCue.Custom("Hi-Hat-Kuppe wird malträtiert")) {
        bar.brightness.sine(2.seconds, 20.percent, 50.percent)
    }

    lightStep(StepCue.Custom("2. Breakdown am Ende")) {
        bar.brightness.fade(100.percent, 200.milliseconds)
        bar.color.fade(Color(0.degrees, saturation = 80.percent), 5.seconds)

        FrontLights.all { it.brightness.off() }
        Washs.both { it.brightness.off(200.milliseconds) }

        launch {
            delay(200)
            Washs.both { it.colorWheelMode.static(CoemarProWash.ColorWheelMode.WarmWhite) }
        }
    }

    step(StepCue.Custom("Schlussschlag")) {
        curtainState = CurtainState.CLOSED

        onRun {
            bar.brightness.pulseOnce(200.milliseconds, 1.seconds)
            FrontLights.all { it.brightness.pulseOnce(0.milliseconds, 3.seconds) }
            Washs.both { it.brightness.pulseOnce(0.milliseconds, 1.seconds) }
        }
    }

    // + Ende in der nächsten Szene
}