package de.moritzruth.lampenfieber.song

import de.moritzruth.lampenfieber.device.*
import de.moritzruth.theaterdsl.show.SceneBuilderContext
import de.moritzruth.theaterdsl.show.StepCue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.degrees
import de.moritzruth.theaterdsl.value.percent
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import kotlin.time.Duration.Companion.milliseconds
import kotlin.time.Duration.Companion.minutes
import kotlin.time.Duration.Companion.seconds

private fun SceneBuilderContext.intro() {
    lightStep(StepCue.Custom("Erster Klavier-Ton")) {
        Washs.both.forEach { it.brightness.pulseOnce(200.milliseconds, 2000.milliseconds) }
    }

    repeat(10) {
        lightStep(StepCue.Custom("Nächster Klavier-Ton")) {
            Washs.both.forEach { it.brightness.pulseOnce(200.milliseconds, 2000.milliseconds) }
        }
    }

    lightStep(StepCue.Custom("Nächster Klavier-Ton")) {
        Washs.both.forEach { it.brightness.fade(100.percent, 200.milliseconds) }
    }

    lightStep(StepCue.Custom("5× gleicher Klavier-Ton")) {
        Washs.both.forEach {
            it.pointAtGuests()
            it.brightness.fade(0.percent, 1.seconds)
        }
    }
}

@Suppress("DuplicatedCode")
fun SceneBuilderContext.songRopeSkippingTanz() {
    // intro()

    step(StepCue.MusicStart("Rope-Skipping-Tanz", 3.minutes)) {
        onRun {
            Washs.both.forEach {
                it.pointAtCeiling()
                it.colorWheelMode.static(CoemarProWash.ColorWheelMode.WarmWhite)
            }
        }
    }

    lightStep(StepCue.Custom("Schlagzeug-Einsatz")) {
        Washs.both.forEach {
            it.brightness.static(100.percent)
            it.strobeSpeed.static(50.percent)
        }
    }

    lightStep(StepCue.Custom("Nächster Takt-Anfang")) {
        Washs.both.forEach {
            it.brightness.static(100.percent)
            it.strobeSpeed.static(0.percent)
            it.pointAtCeiling()
        }

        Washs.left.colorWheelMode.static(CoemarProWash.ColorWheelMode.Rotate(30.percent, false))
        Washs.right.colorWheelMode.static(CoemarProWash.ColorWheelMode.Rotate(30.percent, true))
    }

    lightStep(StepCue.Custom("Crash-Becken, Klavier-Solo")) {
        Washs.both.forEach { it.brightness.off(200.milliseconds) }
    }

    lightStep(StepCue.Custom("+2 Takte")) {
        FrontLights.all.forEach { it.brightness.fade(50.percent, 30.seconds) }
        bar.brightness.sine(4.seconds, 20.percent, 50.percent)
        bar.color.static(Color(40.degrees, 25.percent))

        Washs.both.forEach {
            it.brightness.fade(100.percent, 200.milliseconds)
        }
    }

    lightStep(StepCue.Custom("Crash-Becken, Klavier-Solo")) {
        Washs.both.forEach { it.brightness.fade(0.percent, 200.seconds) }
    }

    lightStep(StepCue.Custom("+1 Takt")) {
        Washs.both.forEach {
            it.pointAtGuests()
            it.brightness.fade(100.percent, 500.milliseconds)
            it.colorWheelMode.static(CoemarProWash.ColorWheelMode.WarmWhite)
        }
    }

    lightStep(StepCue.Custom("Schlagzeug-Einsatz")) {
        Washs.both.forEach {
            it.brightness.static(100.percent)
            it.strobeSpeed.static(50.percent)
        }
    }

    lightStep(StepCue.Custom("Nächster Takt-Anfang")) {
        Washs.both.forEach {
            it.brightness.static(100.percent)
            it.strobeSpeed.static(0.percent)
            it.pointAtCeiling()
        }
    }

    lightStep(StepCue.Custom("Schlagzeug-Einsatz")) {
        Washs.left.colorWheelMode.static(CoemarProWash.ColorWheelMode.Rotate(30.percent, false))
        Washs.right.colorWheelMode.static(CoemarProWash.ColorWheelMode.Rotate(30.percent, true))
    }

    lightStep(StepCue.Custom("Hi-Hat-Kuppe wird malträtiert")) {
        bar.brightness.sine(2.seconds, 20.percent, 50.percent)
    }

    lightStep(StepCue.Custom("Breakdown am Ende")) {
        bar.brightness.fade(25.percent, 200.milliseconds)
        FrontLights.all.forEach { it.brightness.off() }
        Washs.both.forEach { it.brightness.off(200.milliseconds) }

        launch {
            delay(200)
            Washs.both { it.colorWheelMode.static(CoemarProWash.ColorWheelMode.WarmWhite) }
        }
    }

    lightStep(StepCue.Custom("Schlussschlag")) {
        bar.brightness.pulseOnce(200.milliseconds, 2.seconds)
        Tops.both { it.brightness.off(5.seconds) }
        FrontLights.all { it.brightness.pulseOnce(0.milliseconds, 2.seconds) }
        Washs.both { it.brightness.pulseOnce(0.milliseconds, 2.seconds) }
    }

    // + Ende in der nächsten Szene
}