package de.moritzruth.lampenfieber.song

import de.moritzruth.lampenfieber.device.*
import de.moritzruth.theaterdsl.device.reset
import de.moritzruth.theaterdsl.show.CurtainState
import de.moritzruth.theaterdsl.show.SceneBuilderContext
import de.moritzruth.theaterdsl.show.StepCue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.degrees
import de.moritzruth.theaterdsl.value.percent
import kotlinx.coroutines.delay
import kotlinx.coroutines.isActive
import kotlinx.coroutines.launch
import kotlin.random.Random
import kotlin.time.Duration.Companion.milliseconds
import kotlin.time.Duration.Companion.minutes
import kotlin.time.Duration.Companion.seconds

fun SceneBuilderContext.songSchauMichMit() {
    step(StepCue.MusicStart("Schau mich mit anderen Augen an", 3.minutes + 40.seconds)) {
        onRun {
            sideLight.moveColorsPinkAndBlue()

            Washs.left.run {
                brightness.switch(0.percent, 100.percent, 1.seconds, false)
                magenta.sine(5.seconds, start = 100.percent)
                cyan.sine(10.seconds, start = 100.percent)
            }

            Washs.right.run {
                brightness.switch(100.percent, 0.percent, 1.seconds, false)
                magenta.sine(5.seconds, start = 0.percent)
                cyan.sine(10.seconds, start = 0.percent)
            }

            Tops.both { it.brightness.fade(100.percent, 1.seconds) }
            FrontLights.all { it.brightness.fade(40.percent, 10.seconds) }
        }
    }

    lightStep(StepCue.Custom("Bridge")) {
        bar.color.static(Color(50.degrees))
        bar.brightness.fade(100.percent, 200.milliseconds)

        sideLight.colors.forEach { it.fade(Color(50.degrees), 100.milliseconds) }

        Washs.both { it.brightness.off() }
        Tops.both { it.startRoomMovement(2.0) }
    }

    lightStep(StepCue.Custom("Refrain")) {
        bar.color.fadeRandomAround(50.degrees, 10.degrees, 2.seconds)
        bar.brightness.fade(100.percent, 200.milliseconds)

        Tops.both { it.brightness.sine(8.seconds, minimum = 50.percent) }

        sideLight.colors.forEach {
            launch {
                val downDuration = 1000.milliseconds
                val upDuration = 500.milliseconds

                while (isActive) {
                    delay(Random.nextInt(0, 3000).milliseconds)

                    it.fade(Color(50.degrees, brightness = 0.percent), downDuration)
                    delay(downDuration)

                    it.fade(Color(50.degrees, brightness = 100.percent), upDuration)
                    delay(upDuration)
                }
            }
        }
    }

    lightStep(StepCue.Custom("Strophe (mit dem ersten Wort)")) {
        val interval = 800.milliseconds

        bar.brightness.switch(0.percent, 100.percent, interval, false)

        sideLight.brightness.switch(100.percent, 0.percent, interval, false)
        sideLight.colors.forEach { it.static(Color(60.degrees, saturation = 10.percent)) }

        Washs.both { it.strobeSpeed.static(70.percent) }
        Tops.both { it.brightness.off(200.milliseconds) }
    }

    // 3:06
    lightStep(StepCue.Custom("Pause nach »rotiere«")) {
        bar.brightness.off(2.seconds)
        sideLight.brightness.off(2.seconds)
    }

    step(StepCue.Custom("Refrain")) {
        rightSpotTarget = "Tina und Sandra (rechtzeitig!)"

        onRun {
            sideLight.moveColorsPinkAndBlue()
            sideLight.brightness.fade(100.percent, 200.milliseconds)

            bar.color.fadeRandomAround(50.degrees, 10.degrees, 2.seconds)
            bar.brightness.fade(100.percent, 200.milliseconds)

            Washs.both { it.brightness.static(75.percent) }
            Tops.both {
                it.startRoomMovement(8.0)
                it.brightness.static(100.percent)
            }
        }
    }

    lightStep(StepCue.Custom("Steigerung zum Schlussschlag")) {
        FrontLights.all { it.brightness.off() }
    }

    // 3:36
    lightStep(StepCue.Custom("Schlussschlag")) {
        spotRight.brightness.fade(start = 100.percent, end = 0.percent, duration = 5.seconds)
        sideLight.brightness.off(2.seconds)
        bar.brightness.off(200.milliseconds)

        Tops.both { it.brightness.off(200.milliseconds) }
        Washs.both { it.strobeSpeed.off() }
    }

    step(StepCue.MusicEnd) {
        actors {
            -"Tina / nach links"
            -"Sandra / nach rechts"
        }

        curtainState = CurtainState.CLOSED
        rightSpotTarget = null

        onRun {
            Washs.both { it.reset() }
            Tops.both { it.pointAtStageMiddleSingleCenterMarking() }
        }
    }
}