package de.moritzruth.theaterdsl.device

import de.moritzruth.theaterdsl.dmx.DmxAddress
import de.moritzruth.theaterdsl.dmx.DmxDataWriter
import kotlinx.collections.immutable.ImmutableCollection
import kotlinx.collections.immutable.ImmutableSet
import kotlinx.collections.immutable.toImmutableSet

interface Device {
    val dvs: ImmutableSet<DynamicValue<*>>

    val firstChannel: DmxAddress
    val numberOfChannels: UInt
    fun writeDmxData(writer: DmxDataWriter, isLightBehindCurtainOn: Boolean)
}

fun Device.reset() {
    dvs.forEach(DynamicValue<*>::reset)
}

@JvmInline
value class DeviceGroup<T : Device> private constructor(val devices: ImmutableSet<T>) : ImmutableSet<T> by devices {
    operator fun invoke(block: (T) -> Unit) {
        devices.forEach(block)
    }

    operator fun plus(other: ImmutableCollection<T>) = DeviceGroup((other + devices).toImmutableSet())

    fun reset() = devices.forEach(Device::reset)

    companion object {
        operator fun <T : Device> invoke(vararg devices: T) = DeviceGroup(devices.toSet().toImmutableSet())
    }
}