package de.moritzruth.theaterdsl.dmx

import de.moritzruth.theaterdsl.value.Percentage

interface DmxDataWriter {
    fun writeRaw(value: DmxValue)

    fun writePercentage(value: Percentage) = writeRaw(value.roundToDmxValue())

    /**
     * @param startAtOne Whether the written value is in `1..255` or in `0..255`.
     */
    fun writeInRange(range: ClosedFloatingPointRange<Double>, value: Double, startAtOne: Boolean = false) =
        writeRaw(Percentage((value - range.start) / (range.endInclusive - range.start)).roundToDmxValue(startAtOne))

    fun writeHighByte(value: UShort) = writeRaw(DmxValue(value.toUInt().shr(8).toUByte()))
    fun writeLowByte(value: UShort) = writeRaw(DmxValue(value.toUByte()))
}
