package de.moritzruth.theaterdsl.dmx

import com.fazecast.jSerialComm.SerialPort
import com.fazecast.jSerialComm.SerialPortIOException
import io.github.oshai.KotlinLogging
import kotlin.concurrent.thread

object EnttecOpenDmxUsb {
    private const val PORT_NAME = "FT232R USB UART"

    private val logger = KotlinLogging.logger("EnttecOpenDmxUsb")

    var data = UByteArray(512)
        set(value) {
            require(value.size <= 512) { "array length <= 512" }
            field = value
        }

    private val thread: Thread = thread(
        isDaemon = true,
        name = "DMX sender",
        priority = Thread.NORM_PRIORITY - 1,
        start = false
    ) {
        while (Thread.currentThread().isAlive) {
            tryConnection()
            Thread.sleep(1000)
        }
    }

    fun start() {
        thread.start()
    }

    private fun tryConnection() {
        val port = SerialPort.getCommPorts().find { it.portDescription == PORT_NAME }

        if (port !== null) {
            val isSuccess = setupPort(port)
            if (isSuccess) {
                loopSend(port)
            }
        }
    }

    fun loopSend(port: SerialPort) {
        Thread.currentThread().priority = Thread.NORM_PRIORITY + 1
        val stream = port.outputStream

        try {
            while (Thread.currentThread().isAlive) {
                if (!port.isOpen) return

                port.clearRTS()
                port.setDTR()
                port.setBreak()
                Thread.sleep(0, 92000)
                port.clearRTS()
                port.setDTR()
                port.clearBreak()
                Thread.sleep(0, 12000)
                stream.write(0)
                stream.write(data.toByteArray())
                stream.flush()

                Thread.sleep(23)
            }
        } catch (exception: SerialPortIOException) {
            // only thrown when the program is stopped
        }
    }

    fun setupPort(port: SerialPort): Boolean {
        if (port.openPort()) {
            port.baudRate = 250000
            port.clearRTS()
            port.setDTR()
            port.numDataBits = 8
            port.parity = SerialPort.NO_PARITY
            port.numStopBits = SerialPort.TWO_STOP_BITS
            port.setFlowControl(SerialPort.FLOW_CONTROL_DISABLED)

            logger.info("DMX interface connected")
            return true
        } else {
            if (port.lastErrorCode == 13) logger.error("The serial port could not be opened because of insufficient permissions (13).")
            else logger.error("The serial port could not be opened (${port.lastErrorCode}).")
            return false
        }
    }
}
